def numtiled(word, tiles):
    # Calculate N(i), the maximum number of tiled letters starting
    # at index i.
    # N(i) = N(i+1) if no tile starts at i
    #        max(len(tile) + N(i+len(tile)), N(i+1)) if a tile starts at i
    tiles = tuple(tiles)
    def N(i, _cache={}):
        if i in _cache: return _cache[i]
        if i >= len(word): return 0
        res = N(i+1)
        for tile in tiles:
            l = len(tile)
            if word[i:i+l] == tile:
                res = max(res, l + N(i+l))
        _cache[i] = res
        return res
    return N(0)

# 50 US states
states = "al ak az ar ca co ct de fl ga hi id il in ia ks ky la me md ma mi mn ms mo mt ne nv nh nj nm ny nc nd oh ok or pa ri sc sd tn tx ut vt va wa wv wi wy".split()

# 112 chemical elements with atomic number 112 and below
elements = "ac ag al am ar as at au b ba be bh bi bk br c ca cd ce cf cl cm cn co cr cs cu db ds dy er es eu f fe fm fr ga gd ge h he hf hg ho hs i in ir k kr la li lr lu md mg mn mo mt n na nb nd ne ni no np o os p pa pb pd pm po pr pt pu ra rb re rf rg rh rn ru s sb sc se sg si sm sn sr ta tb tc te th ti tl tm u v w xe y yb zn zr".split()

# 249 alpha-2 country codes officially assigned in ISO 3166-1
countries = "ad ae af ag ai al am ao aq ar as at au aw ax az ba bb bd be bf bg bh bi bj bl bm bn bo bq br bs bt bv bw by bz ca cc cd cf cg ch ci ck cl cm cn co cr cu cv cw cx cy cz de dj dk dm do dz ec ee eg eh er es et fi fj fk fm fo fr ga gb gd ge gf gg gh gi gl gm gn gp gq gr gs gt gu gw gy hk hm hn hr ht hu id ie il im in io iq ir is it je jm jo jp ke kg kh ki km kn kp kr kw ky kz la lb lc li lk lr ls lt lu lv ly ma mc md me mf mg mh mk ml mm mn mo mp mq mr ms mt mu mv mw mx my mz na nc ne nf ng ni nl no np nr nu nz om pa pe pf pg ph pk pl pm pn pr ps pt pw py qa re ro rs ru rw sa sb sc sd se sg sh si sj sk sl sm sn so sr ss st sv sx sy sz tc td tf tg th tj tk tl tm tn to tr tt tv tw tz ua ug um us uy uz va vc ve vg vi vn vu wf ws ye yt za zm zw".split()

def main():
    import sys
    with open("words.txt") as fp:
        words = set(fp.read().splitlines())

    # you can empty this to simulate solving without knowing the keywords are valid
    # solutions to puzzles, but you'll need to remove the now-stale tiles.pickle when
    # you fill it up again
    keywords = set([
            "djoser", "sekhemkhet", "khaba", "sneferu", "khufu",
            "djedefre", "khafra", "menkaure", "userkaf", "sahure",
            "neferefre", "niuserre", "unas", "teti", "pepi",
            "merenre", "khui", "merikare", "amenemhat", "senusret",
            "khendjer", "bikheris", "sobekneferu", "ahmose",
            "saqqara", "dahshur", "mazghuna", "abusir",
            "meidum", "lisht", "hawara", "nuri", "giza", "illahun",
            "sethka", "djedkareisesi", "qakareibi", "amenyqemau",
    ])
    allwords = words | keywords

    global wl3
    wl3 = [w for w in words if len(w) <= 3]

    import collections
    allvalues = collections.defaultdict(dict)

    for setname in ("states", "elements", "countries", "wl3"):
        setcontents = globals()[setname]
        chunksize = len(allwords) / 100
        values = allvalues[setname]
        for i, word in enumerate(allwords):
            if i % chunksize == 0:
                print "Caching tiles for %s: %.1f%% done\r" % (
                    setname, i*100.0/len(allwords)),
                sys.stdout.flush()
            values[word] = numtiled(word, setcontents)
        print "Caching tiles for %s: done        " % (setname,)

    with open("tiles.pickle", "w") as fp:
        import cPickle
        cPickle.dump(allvalues, fp)

if __name__ == "__main__":
    main()
